import pandas as pd
from datetime import datetime
import numpy as np


def convert_time_str_to_datetime(time_str_series):
    # find first non nan value
    idx = 0
    for i in range(len(time_str_series)):
        if type(time_str_series[i]) == str:
            idx = i
            break
    else:
        return time_str_series

    if len(time_str_series[idx].split(".")) > 1:
        converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S.%f") if (type(x) == str) else x
    else:
        converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S") if (type(x) == str) else x
    time_datetime_list = [x.split(" ")[0] + " " + x.split(" ")[1] if type(x) == str else x for x in
                          list(time_str_series)]

    time_datetime_list = pd.Series(map(converter, time_datetime_list))

    return time_datetime_list


def create_column(feature_df, parts_num):
    print("\n> start generating the feature: Parts of waking hours")
    wake_time_datetime_list = convert_time_str_to_datetime(feature_df["WAKE_TIME"])
    sleep_time_datetime_list = convert_time_str_to_datetime(feature_df["SLEEP_TIME"])
    prompt_time_datetime_list = feature_df["Prompt_Local_DateTime"]

    proximity_to_wake_time_column = [
        (y - x).total_seconds() / 60 if (type(x) == float) or (type(y) == float) else np.nan for x, y in
        zip(wake_time_datetime_list, prompt_time_datetime_list)]
    proximity_to_sleep_time_column = [
        (y - x).total_seconds() / 60 if (type(x) == float) or (type(y) == float) else np.nan for x, y in
        zip(prompt_time_datetime_list, sleep_time_datetime_list)]
    wake_duration = [(y - x).total_seconds() / 60 if (type(x) == float) or (type(y) == float) else np.nan for x, y in
                     zip(wake_time_datetime_list, sleep_time_datetime_list)]
    duration_per_part = [x // parts_num if type(x) == float else np.nan for x in wake_duration]
    parts_of_waking_hour_column = [y // x if (type(x) == float) or (type(y) == float) else np.nan for x, y in
                                   zip(duration_per_part, proximity_to_wake_time_column)]

    print("     --- success")

    return parts_of_waking_hour_column, proximity_to_wake_time_column, proximity_to_sleep_time_column



"""
Output: 1) a list of targeted features for the participant 2) print stats of the targeted features for the participant
"""

from os import path, sep, listdir
import pandas as pd
from datetime import datetime
from collections import Counter


def print_stats_single_column(feature_name, data):
    print('--------- Stats of Feature: ' + feature_name + ' ---------')
    counter = Counter(data)
    print(sorted(counter.items()))


def calculate_daysInStudy(dateobject, start_date_dict, name):
    start_date = start_date_dict[name]
    delta = dateobject - start_date
    return delta

# def get_start_date_dict():
#     # change to your path of combined_report.csv
#     combined_report_path = r"E:\compliance_analysis\report\combined_report.csv"
#     df_report = pd.read_csv(combined_report_path)
#     participants_list = df_report['participant_ID'].unique()
#     date_format = '%Y-%m-%d'
#     start_date_dict = {}
#     for p_id in participants_list:
#         start_date_str = list(df_report[df_report['participant_ID'] == p_id]['start_date'])[0]
#         start_date = datetime.strptime(start_date_str, date_format).date()
#         start_date_dict[p_id] = start_date
#     return start_date_dict

def get_start_date_dict():
    # change to your path of combined_report.csv
    combined_report_path = r"E:\compliance_analysis\report\combined_report.csv"
    # combined_report_path = r"C:\Users\jixin\Desktop\missing\start_date_table.csv"
    df_report = pd.read_csv(combined_report_path)
    participants_list = df_report['participant_ID'].unique()
    date_format = '%m/%d/%Y'
    start_date_dict = {}
    for p_id in participants_list:
        start_date_str = list(df_report[df_report['participant_ID'] == p_id]['start_date'])[0]
        start_date = datetime.strptime(start_date_str, date_format).date()
        start_date_dict[p_id] = start_date
    return start_date_dict

def create_column(df_prompt_combined, print_stats=True):
    print("\n> start generating the feature: days in study ")
    # get the start date dictionary
    start_date_dict = get_start_date_dict()

    # map from timestamp to date object
    converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S.%f")
    prompt_dateobject_column = pd.Series(map(converter, df_prompt_combined['Prompt_Local_Time']))
    # prompt_dateobject_column = pd.Series(
    #     map(lambda x: datetime.fromtimestamp(x / 1000.0), df_prompt_combined['Prompt_Timestamp']))
    # get start_date dateobject
    df = pd.DataFrame(
        {"Participant_ID": list(df_prompt_combined['Participant_ID']), "Date": prompt_dateobject_column.dt.date})
    group_df = df.groupby("Participant_ID")
    time_in_study = group_df.apply(lambda x: calculate_daysInStudy(x.Date, start_date_dict, x.name))
    date_list = list(time_in_study.reset_index().sort_values(by=["level_1"])['Date'])
    days_in_study = [x.days for x in date_list]

    if print_stats == True:
        # print_stats_single_column("daysInStudy", prompt_daysInStudy_column)
        pass
    return days_in_study



import os
from os import sep
from glob import glob
import pandas as pd
import numpy as np
# from tqdm import tqdm
from datetime import datetime, timedelta
from microt_compliance_matrix import utils


target_file_pattern = 'phone_system_events_clean*.csv'

def validate_dates_before_after(intermediate_participant_path, date_in_study):
    validated_date_list = []

    # target date
    date_folder_path = intermediate_participant_path + sep + date_in_study
    target_date_log_paths = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name
    if len(target_date_log_paths) == 0:
        print("No battery daily file on {}".format(date_in_study))
    else:

        # 1 day before target date
        date_format = "%Y-%m-%d"
        one_date_before_datetime = datetime.strptime(date_in_study, date_format).date() - timedelta(days=1)
        one_date_before = one_date_before_datetime.strftime(date_format)
        date_folder_path = intermediate_participant_path + sep + one_date_before
        one_day_before_log_paths = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name
        if len(one_day_before_log_paths) != 0:
            validated_date_list.append(one_date_before)

        # target date
        validated_date_list.append(date_in_study)

        # 1 day after target date
        date_format = "%Y-%m-%d"
        one_date_after_datetime = datetime.strptime(date_in_study, date_format).date() + timedelta(days=1)
        one_date_after = one_date_after_datetime.strftime(date_format)
        date_folder_path = intermediate_participant_path + sep + one_date_after
        one_day_after_log_paths = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name
        if len(one_day_after_log_paths) != 0:
            validated_date_list.append(one_date_after)


    return validated_date_list

def combine_intermediate_file(intermediate_participant_path, date_in_study):
    df_logs_combined = pd.DataFrame()
    participant_id = utils.extract_participant_id.extract_participant_id(intermediate_participant_path)

    # generate date range where date folder exists (sharable code in utils)
    validated_date_list = validate_dates_before_after(intermediate_participant_path, date_in_study)
    if len(validated_date_list) == 0:
        print("Cannot find logs file around {}".format(date_in_study))
        return df_logs_combined

    for date in validated_date_list:
        date_folder_path = intermediate_participant_path + sep + date
        csv_path_list = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name

        csv_path = csv_path_list[0]
        try:
            df_day = pd.read_csv(csv_path)
        except pd.errors.EmptyDataError:
            raise Exception("pandas.errors.EmptyDataError (screen_status) : " + csv_path)
        except pd.errors.ParseError:
            raise Exception("pandas.errors.ParseError (screen_status) : " + csv_path)

        if df_day.shape[0] > 0:
            df_day['Participant_ID'] = [participant_id] * df_day.shape[0]
            df_logs_combined = pd.concat([df_logs_combined, df_day])

    # filter out irrelevant logs
    df_logs_combined = df_logs_combined[df_logs_combined['PHONE_EVENT'].isin(["PHONE_SCREEN_ON","PHONE_SCREEN_OFF"])]
    df_logs_combined.reset_index(inplace=True, drop=True)

    converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S.%f")
    df_logs_combined = df_logs_combined.dropna(subset=['LOG_TIME'])
    df_logs_combined.reset_index(inplace=True, drop=True)
    df_logs_combined["LOG_TIME"] = [x.split(" ")[0] + " " + x.split(" ")[1] for x in
                                         list(df_logs_combined["LOG_TIME"])]

    df_logs_combined['LOG_TIME_DATETIME'] = pd.Series(map(converter, df_logs_combined["LOG_TIME"]))
    if df_logs_combined.shape[0] > 0:
        df_logs_combined['Date'] = df_logs_combined['LOG_TIME_DATETIME'].dt.date

    return df_logs_combined


# def find_closest_time(prompt_time, subset_time_list):
#     i = bisect.bisect_left(subset_time_list, prompt_time)
#     closet_time = min(subset_time_list[max(0, i - 1): i + 2], key=lambda t: abs(prompt_time - t))
#     return closet_time
def find_prev_closest_time(prompt_time, subset_time_series):
    previous_timestamps = subset_time_series[subset_time_series < prompt_time]
    reverse = False
    if len(previous_timestamps) > 0:
        closet_time = previous_timestamps.max()
    else:
        after_timestamps = subset_time_series[subset_time_series > prompt_time]
        closet_time = after_timestamps.min()
        reverse = True
    return closet_time, reverse


def match_feature(prompt_local_datetime_series, df_logs_combined):
    print("     --- start matching")
    matched_result_list = []
    matched_time_list = []

    for idx in prompt_local_datetime_series.index:
        prompt_time = prompt_local_datetime_series[idx]
        # prompt_date = prompt_time.date()


        if df_logs_combined.shape[0] == 0:
            screen_status = np.nan
            closest_time = np.nan
        else:
            subset_time_series = df_logs_combined["LOG_TIME_DATETIME"]

            closest_time, reverse = find_prev_closest_time(prompt_time, subset_time_series)

            screen_event = list(df_logs_combined[df_logs_combined['LOG_TIME_DATETIME'] == closest_time][
                                     "PHONE_EVENT"])[0]

            # to determine screen on/off
            if not reverse:
                if screen_event == "PHONE_SCREEN_ON":
                    screen_status = "Screen On"
                elif screen_event == "PHONE_SCREEN_OFF":
                    screen_status = "Screen Off"
                else:
                    screen_status = np.nan
            else:
                if screen_event == "PHONE_SCREEN_ON":
                    screen_status = "Screen Off"
                elif screen_event == "PHONE_SCREEN_OFF":
                    screen_status = "Screen On"
                else:
                    screen_status = np.nan

        matched_time_list.append(closest_time)
        matched_result_list.append(screen_status)

    return matched_result_list, matched_time_list


def transform(screen_status_column):
    # return [1 if x >= 15 else 0 for x in screen_status_column]
    return screen_status_column


def create_column(prompt_local_datetime_series, intermediate_participant_path, date_in_study):
    print("\n> start generating the feature: Screen status ")

    # Read, parse and combine related intermediate file
    df_logs_combined = combine_intermediate_file(intermediate_participant_path, date_in_study)

    if df_logs_combined.shape[0] > 0:
        # Match the combined parsed intermediate file with prompt feature data frame
        result_column, match_time_screen = match_feature(prompt_local_datetime_series, df_logs_combined)
    else:
        result_column = ["NF"] * len(prompt_local_datetime_series)
        match_time_screen = ["NF"] * len(prompt_local_datetime_series)

    # transform feature
    result_column_transformed = transform(result_column)
    print("     --- success")

    return result_column_transformed, match_time_screen



from .preprocess_promptResponse_raw import *
from .combine_appended_response import *


def create_combined_csv_file(participant_ids, intermediate_root_path, feature_save_path, start_date, end_date):
    for p_id in participant_ids:
        print("Now processing PromptResponse.csv file for {}".format(p_id))
        create_temporary_appended_response_files(p_id, intermediate_root_path, feature_save_path, start_date, end_date)

    df_prompt = combine_appended_response_for_all_participants(participant_ids, feature_save_path)
    return df_prompt



"""
Output: 1) a list of targeted features for the participant 2) print stats of the targeted features for the participant
"""

from datetime import datetime



def calculate_daysInStudy(prompte_date, start_date):
    delta = prompte_date - start_date
    return delta

# def get_start_date_dict():
#     # change to your path of combined_report.csv
#     combined_report_path = r"E:\compliance_analysis\report\combined_report.csv"
#     df_report = pd.read_csv(combined_report_path)
#     participants_list = df_report['participant_ID'].unique()
#     date_format = '%m/%d/%Y'
#     start_date_dict = {}
#     for p_id in participants_list:
#         start_date_str = list(df_report[df_report['participant_ID'] == p_id]['start_date'])[0]
#         start_date = datetime.strptime(start_date_str, date_format).date()
#         start_date_dict[p_id] = start_date
#     return start_date_dict

def create_column(df_prompt_combined, info_dict, print_stats=True):
    print("\n> start generating the feature: days in study ")
    participant_id = df_prompt_combined['Participant_ID'][0]

    if participant_id not in info_dict:
        raise Exception("Don't find this participant in info json. Check if this participant has data, and delete Misc and rerun the code. ")
    else:
        if "start_date" in info_dict[participant_id]:
            start_date = info_dict[participant_id]["start_date"]
            start_date_dateobject = datetime.strptime(start_date, "%Y-%m-%d").date()
        else:
            raise Exception(
                "Don't find this participant in info json. Check if this participant has daily report, and delete Misc and rerun the code. ")


    prompt_dateobject_date_series = df_prompt_combined.Prompt_Local_DateTime.dt.date

    days_in_study = prompt_dateobject_date_series.apply(lambda x: calculate_daysInStudy(x, start_date_dateobject))

    return days_in_study



from datetime import datetime, timedelta
import pandas as pd
import numpy as np

nat = np.datetime64('NaT')


def parse_time_offset(time_offset):
    sign_str = time_offset.strip('GMT')[0]
    if sign_str == "-":
        sign = -1
    elif sign_str == "+":
        sign = 1
    else:
        sigh = 0

    time_offset_int = int(time_offset.strip('GMT')[1:3])
    time_delta = timedelta(hours=time_offset_int)

    return time_delta, sign


def get_time_offset(time_offset):
    time_delta = timedelta(hours=0)
    sign = 0

    time_delta, sign = parse_time_offset(time_offset)

    return time_delta, sign


def convert_timestamp_int_list_to_readable_time(timestamp_int_list, time_offset):
    time_delta, sign = get_time_offset(time_offset)
    if sign == 0:
        readable_time_str = ["unknown time zone"] * len(timestamp_int_list)
    else:
        # print(timestamp_int_list)
        timestamp_int_list_nat = timestamp_int_list.replace({"-1": nat})
        # timestamp_int_list_nat = [nat if x == -1 else x for x in list(timestamp_int_list)]
        timestamp_naive_list = pd.to_datetime(timestamp_int_list_nat, unit='ms', errors='ignore')
        # print(timestamp_naive_list)
        converter = lambda x: x + sign * time_delta if not pd.isnull(x) else x
        timestamp_TZaware_list = list(map(converter, timestamp_naive_list))
        translator = lambda x: x.strftime("%Y-%m-%d %H:%M:%S.%f")[:-3] if not pd.isnull(x) else x
        readable_time_str = pd.Series(map(translator, timestamp_TZaware_list))
        readable_time_str.replace({nat: "-1"}, inplace=True)
    return readable_time_str


if __name__ == "__main__":
    sample_csv_path = r"E:\compliance_analysis\prompt_response\aditya4_internal@timestudy_com\appended_response\aditya4_internal@timestudy_com_uEMA_2020-06-07.csv"
    time_zone = "GMT-04:00"
    df = pd.read_csv(sample_csv_path)
    print(convert_timestamp_int_list_to_readable_time(df['Response_Timestamp'], time_zone))
    # print("{} is translated to {} .".format(str(timestamp_int), convert_timestamp2string(timestamp_int, time_zone)))



from os import path, sep, makedirs
from glob import glob
import pandas as pd
import numpy as np
import bisect
import requests
from tqdm import tqdm
import pickle

from microt_compliance_matrix.utils.validate_date import *


def combine_intermediate_file(participants_ids, intermediate_root_path, feature_save_path, start_date, end_date):
    df_combined = pd.DataFrame()
    for p_id in participants_ids:
        participant_logs_path = intermediate_root_path + sep + "intermediate_file" + sep + p_id

        # step 1: generate date range where date folder exists (sharable code in utils)
        validated_date_list = validate_date(participant_logs_path, start_date, end_date)
        if len(validated_date_list) == 0:
            print("Cannot find date folder in data source between {} and {}".format(start_date, end_date))
            return None

        for date in validated_date_list:
            date_folder_path = participant_logs_path + sep + date
            csv_path_list = sorted(glob(os.path.join(date_folder_path, 'phone_GPS_clean*.csv')))  # file name
            if len(csv_path_list) == 0:
                print("No GPS daily file on {}".format(date))
                continue

            csv_path = csv_path_list[0]
            df_daily = pd.read_csv(csv_path)
            if df_daily.shape[0] > 0:
                df_daily['Participant_ID'] = [p_id] * df_daily.shape[0]
                df_combined = pd.concat([df_combined, df_daily])

    # save a copy of combined file in compliance anaysis folder
    save_path = feature_save_path + sep + "compliance_analysis" + sep + "uEMA_plus" + sep + "location"
    if df_combined.shape[0] > 0:
        if not path.exists(save_path):
            makedirs(save_path)
        df_combined.reset_index(inplace=True, drop=True)
        df_combined.to_csv(save_path + sep + "combined.csv", index=False)

    return df_combined


def find_closest_time(prompt_time, subset_time_list):
    i = bisect.bisect_left(subset_time_list, prompt_time)
    closet_time = min(subset_time_list[max(0, i - 1): i + 2], key=lambda t: abs(prompt_time - t))
    return closet_time


def match_feature(feature_save_path, prompt_feature_df, df_gps_combined):
    print("     --- start transformation\n")
    converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S.%f")
    prompt_feature_df['PROMPT_TIMESTAMP'] = pd.Series(map(converter, prompt_feature_df['PROMPT_LOCAL_TIME']))
    df_gps_combined["LOCATION_TIME"] = [x.strip(x.split(' ')[2]).strip(" ") for x in
                                        list(df_gps_combined["LOCATION_TIME"])]
    converter2 = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S")
    df_gps_combined['LOCATION_TIMESTAMP'] = pd.Series(map(converter2, df_gps_combined["LOCATION_TIME"]))
    df_gps_combined['Date'] = df_gps_combined['LOCATION_TIMESTAMP'].dt.date

    print("     --- start matching\n")
    matched_info_list = []
    for idx in tqdm(prompt_feature_df.index):
        p_id = prompt_feature_df['Participant_ID'][idx]
        prompt_time = prompt_feature_df['PROMPT_TIMESTAMP'][idx]
        prompt_date = prompt_time.date()

        df_gps_participant = df_gps_combined[df_gps_combined['Participant_ID'] == p_id][
            df_gps_combined['Date'] == prompt_date]
        if df_gps_participant.shape[0] == 0:
            location = np.nan
        else:
            # print(df_gps_participant["START_TIMESTAMP"][df_gps_participant.shape[0]-20])
            subset_time_list = list(df_gps_participant["LOCATION_TIMESTAMP"])

            closest_time = find_closest_time(prompt_time, subset_time_list)
            target_idx = df_gps_participant['LOCATION_TIMESTAMP'] == closest_time
            lat = list(df_gps_participant[target_idx]['LAT'])[0]
            long = list(df_gps_participant[target_idx]['LONG'])[0]
            location = [lat, long]

        matched_info_list.append(location)

        if idx % 1000 == 0:
            matched_save_path = os.path.join(*[feature_save_path, "compliance_analysis", "location", "matched_location_list.txt"])

            with open(matched_save_path, "wb") as fp:
                pickle.dump(matched_info_list, fp)

    return matched_info_list


def reverse_geo_nominatim(lat, long):
    URL = "https://nominatim.openstreetmap.org/reverse?"
    PARAMS = {'format': 'geojson', 'lat': lat, 'lon': long}
    r = requests.get(url=URL, params=PARAMS)
    result = r.json()

    category = result['features'][0]['properties']['category']
    type_loc = result['features'][0]['properties']['type']

    location = [category, type_loc]
    return location


#     # here introduces the OpenStreetMap API
# def transform(location_column):
#     result_list = []
#     for gps_pair in location_column:
#         result = reverse_geo_nominatim(gps_pair[0], gps_pair[1])
#         result_list.append(result)
#         print(result)
#     return result_list
def transform(location_column):
    return location_column


def create_column(prompt_feature_df, intermediate_root_path, feature_save_path, start_date, end_date):
    print("\n> start generating the feature: location ")
    participants_ids = prompt_feature_df['Participant_ID'].unique()

    # Read, parse and combine related intermediate file
    df_combined = combine_intermediate_file(participants_ids, intermediate_root_path, feature_save_path, start_date,
                                            end_date)
    # df_combined = pd.read_csv(r"E:\compliance_analysis\location\combined.csv")
    # Match the combined parsed intermediate file with prompt feature data frame
    location_column = match_feature(feature_save_path, prompt_feature_df, df_combined)

    # transform feature
    location_column_transformed = transform(location_column)

    return location_column_transformed


if __name__ == "__main__":
    gps_list = [[42.340702, -71.117756]]
    print(transform(gps_list))



from os import path, sep, makedirs
from glob import glob
import pandas as pd
import numpy as np
import bisect
from tqdm import tqdm
from microt_compliance_matrix.utils.validate_date import *


def combine_intermediate_file(participants_ids, intermediate_root_path, feature_save_path, start_date, end_date):
    df_combined = pd.DataFrame()
    for p_id in participants_ids:
        print("\n", p_id)
        participant_logs_path = intermediate_root_path + sep + "intermediate_file" + sep + p_id

        # step 1: generate date range where date folder exists (sharable code in utils)
        validated_date_list = validate_date(participant_logs_path, start_date, end_date)
        if len(validated_date_list) == 0:
            print("Cannot find date folder in data source between {} and {}".format(start_date, end_date))
            return None

        for date in validated_date_list:
            date_folder_path = participant_logs_path + sep + date
            csv_path_list = sorted(glob(os.path.join(date_folder_path, 'watch_battery_clean*.csv')))  # file name
            if len(csv_path_list) == 0:
                print("No battery daily file on {}".format(date))
                continue

            csv_path = csv_path_list[0]
            df_daily = pd.read_csv(csv_path)
            if df_daily.shape[0] > 0:
                df_daily['Participant_ID'] = [p_id] * df_daily.shape[0]
                df_combined = pd.concat([df_combined, df_daily])

    # remove error line
    df_combined = df_combined[df_combined['BATTERY_LEVEL'] != "BATTERY_LEVEL"]

    # save a copy of combined file in compliance anaysis folder
    save_path = feature_save_path + sep + "compliance_analysis" + sep + "uEMA_plus" + sep + "battery_level"
    if df_combined.shape[0] > 0:
        if not path.exists(save_path):
            makedirs(save_path)
        df_combined.reset_index(inplace=True, drop=True)
        df_combined.to_csv(save_path + sep + "combined.csv", index=False)

    return df_combined


def find_closest_time(prompt_time, subset_time_list):
    i = bisect.bisect_left(subset_time_list, prompt_time)
    closet_time = min(subset_time_list[max(0, i - 1): i + 2], key=lambda t: abs(prompt_time - t))
    return closet_time


def match_feature(prompt_feature_df, df_battery_combined):
    print("     --- start transformation\n")
    converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S.%f")
    prompt_feature_df['PROMPT_TIMESTAMP'] = pd.Series(map(converter, prompt_feature_df['PROMPT_LOCAL_TIME']))
    df_battery_combined["START_TIME"] = [x.strip(x.split(' ')[2]).strip(" ") for x in list(df_battery_combined["START_TIME"])]
    # df_battery_combined.to_csv(r"E:\compliance_analysis\battery_level\waht.csv")
    # converter2 = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S")
    df_battery_combined['START_TIMESTAMP'] = pd.Series(map(converter, df_battery_combined["START_TIME"]))
    df_battery_combined['Date'] = df_battery_combined['START_TIMESTAMP'].dt.date

    print("     --- start matching\n")
    matched_battery_level_list = []
    matched_charging_status_list = []
    for idx in tqdm(prompt_feature_df.index):
        p_id = prompt_feature_df['Participant_ID'][idx]
        prompt_time = prompt_feature_df['PROMPT_TIMESTAMP'][idx]
        prompt_date = prompt_time.date()

        df_battery_participant = df_battery_combined[df_battery_combined['Participant_ID'] == p_id][df_battery_combined['Date'] == prompt_date]
        if df_battery_participant.shape[0] == 0:
            battery_level = np.nan
            charging_status = np.nan
        else:
            # print(df_battery_participant["START_TIMESTAMP"][df_battery_participant.shape[0]-20])
            subset_time_list = list(df_battery_participant["START_TIMESTAMP"])

            closest_time = find_closest_time(prompt_time, subset_time_list)

            battery_level = list(df_battery_participant[df_battery_participant['START_TIMESTAMP'] == closest_time][
                "BATTERY_LEVEL"])[0]
            charging_status = list(df_battery_participant[df_battery_participant['START_TIMESTAMP'] == closest_time][
                "BATTERY_CHARGING"])[0]

        matched_battery_level_list.append(battery_level)
        matched_charging_status_list.append(charging_status)

    return matched_battery_level_list, matched_charging_status_list


def transform(battery_level_column):
    # return [1 if x >= 15 else 0 for x in battery_level_column]
    return battery_level_column


def create_column(prompt_feature_df, intermediate_root_path, feature_save_path, start_date, end_date):
    print("\n> start generating the feature: battery level ")
    participants_ids = prompt_feature_df['Participant_ID'].unique()

    # Read, parse and combine related intermediate file
    df_combined = combine_intermediate_file(participants_ids, intermediate_root_path, feature_save_path, start_date,
                                            end_date)
    # df_combined = pd.read_csv(r"E:\compliance_analysis\battery_level\combined.csv")
    # Match the combined parsed intermediate file with prompt feature data frame
    battery_level_column, charging_status_column = match_feature(prompt_feature_df, df_combined)

    # transform feature
    battery_level_column_transformed = transform(battery_level_column)

    return battery_level_column_transformed, charging_status_column



"""
Functionality: This is the main script to generate a matrix of features of interest.

Input:
1) intermediate root path
2) feature save path
3) participants included text file path
4) start_date (inclusive)
5) end_date (inclusive)
e.g., python main.py [intermediate_root_path] [feature_save_path] [participants_included_text_file_path] start_date end_date

Output: all_features.csv file that contains all features and is ready for modeling after import.
"""
import sys
import pandas as pd
from os import sep


def get_all_features(participant_ids, intermediate_root_path, feature_save_path, start_date, end_date, parallel_participant):
    # step1: parse and combine prompt response file for each participant
    df_prompt_combined = src.feature_engineering.prompt_response.create_combined_csv.create_combined_csv_file(
        participant_ids,
        intermediate_root_path,
        feature_save_path,
        start_date, end_date)

    # df_prompt_combined = pd.read_csv(r"F:\compliance_analysis\prompt_response\combined.csv")
    prompt_timestamp_column = df_prompt_combined['Prompt_Local_Time']
    # feature engineering related to prompt response file
    print_stats = False

    # - outcome: compliance
    compliance_binary = src.feature_engineering.compliance.create_column(df_prompt_combined['Completion_Status'],
                                                                         print_stats)
    # - feature 1: Day of the week
    feature1 = src.feature_engineering.day_of_the_week.create_column(prompt_timestamp_column, print_stats)
    # - feature 2: Time of the day
    feature2 = src.feature_engineering.time_of_the_day.create_column(prompt_timestamp_column, print_stats)
    # - feature 3: Days in the study
    feature3 = src.feature_engineering.days_in_study.create_column(df_prompt_combined, print_stats)

    # combine features
    prompt_feature_df = pd.DataFrame(
        list(zip(df_prompt_combined['Participant_ID'], df_prompt_combined['Prompt_Local_Time'], compliance_binary,
                 feature1, feature2, feature3)),
        columns=['Participant_ID', 'PROMPT_LOCAL_TIME', 'COMPLIANCE', 'DAY_OF_THE_WEEK', 'TIME_OF_THE_DAY',
                 'DAYS_IN_THE_STUDY'])

    feature_df = prompt_feature_df
    feature_df.to_csv(
        feature_save_path + sep + "compliance_analysis" + sep + "uEMA_plus" + sep + "feature_temp.csv", index=False)

    # # - feature 4: battery level
    # feature_df = pd.read_csv(r"F:\compliance_analysis\uEMA\feature_temp_uema2.csv")
    # feature_df = pd.read_csv("/mnt/f/compliance_analysis/uEMA/feature_temp_combined.csv")
    # feature_df = pd.read_csv("/mnt/e/compliance_analysis/uEMA/feature_wake_combined_type.csv")
    # feature_df = feature_df[feature_df.Participant_ID == parallel_participant]
    # if os.path.exists(feature_save_path + sep + "compliance_analysis" + sep + "uEMA" + sep + "feature_lock_" + parallel_participant + ".csv"):
    #     print("File exists for {}".format(parallel_participant))
    #     return

    # feature_df = feature_df[feature_df.Participant_ID == parallel_participant]
    # feature_df.reset_index(inplace=True, drop=True)
    # feature5, feature6 = feature_engineering.battery_level.create_column(feature_df, intermediate_root_path,
    #                                                            feature_save_path, start_date, end_date)
    # feature_df['battery_level'] = feature5
    # feature_df['charging_status'] = feature6
    #
    # feature_df.to_csv(
    #     feature_save_path + sep + "compliance_analysis" + sep + "uEMA" + sep + "feature_battery_" + parallel_participant + ".csv", index=False)

    # - feature 7: GPS location
    # feature7 = feature_engineering.location.create_column(feature_df, intermediate_root_path,
    #                                                       feature_save_path, start_date, end_date)
    # feature_df['location'] = feature7


    # # - feature 8: Home or other locations
    # # home_coordinates_dict = utils.home_coordinates.get_home_boarder_coordinates(participant_ids, intermediate_root_path)
    # # with open(feature_save_path + sep + "compliance_analysis" + sep + "location" + sep + "home_coord_dict.json", 'a') as fp:
    # #     json.dump(home_coordinates_dict, fp)
    # # with open(feature_save_path + sep + "compliance_analysis" + sep + "location" + sep + "home_coord_dict.json", 'rb') as fp:
    # #     home_coordinates_dict = json.load(fp)
    # # feature8 = feature_engineering.home_location.create_column(feature_df, home_coordinates_dict)
    # # feature_df['home'] = feature8
    # #
    # save file
    # feature_df.to_csv(
    #     feature_save_path + sep + "compliance_analysis" + sep + "uEMA" + sep + "feature_gps_" + parallel_participant + ".csv", index=False)
    # feature9 = feature_engineering.screen_status.create_column(feature_df, intermediate_root_path,
    #                                                       feature_save_path, start_date, end_date)
    # feature_df['screen_status'] = feature9
    #
    # feature_df.to_csv(
    #     feature_save_path + sep + "compliance_analysis" + sep + "uEMA" + sep + "feature_screen_" + parallel_participant + ".csv", index=False)

    # - feature 10&11: wake and sleep hour
    # feature10, feature11 = feature_engineering.waking_hours.create_column(feature_df, intermediate_root_path,
    #                                                       feature_save_path, start_date, end_date)
    # feature_df['wake_time'] = feature10
    # feature_df['sleep_time'] = feature11
    # - feature 12: Phone lock status
    # feature12, feature13 = feature_engineering.phone_lock.create_column(feature_df, intermediate_root_path,
    #                                                       feature_save_path, start_date, end_date)
    # feature_df['phone_lock'] = feature12
    # feature_df['last_usage'] = feature13
    #
    # feature_df.to_csv(
    #     feature_save_path + sep + "compliance_analysis" + sep + "uEMA" + sep + "feature_lock_" + parallel_participant + ".csv", index=False)

    return


if __name__ == "__main__":
    intermediate_root_path = sys.argv[1]
    feature_save_path = sys.argv[2]
    participant_text_file_path = sys.argv[3]
    start_date = sys.argv[4]
    end_date = sys.argv[5]
    parallel_participant = sys.argv[6]

    participant_ids = []
    # participant_ids = utils.parse_participants.parse_participants(participant_text_file_path)
    df = get_all_features(participant_ids, intermediate_root_path, feature_save_path, start_date, end_date, parallel_participant)
    # df.to_csv(feature_save_path + sep + "compliance_analysis" + sep + "all_features.csv", index=False)
    print("\nFinished")



import numpy as np

def create_column(feature_df, home_coordinates_dict):
    print("\n> start generating the feature: home location ")

    participants_id = feature_df['Participant_ID'].unique()
    home_list = []
    for p_id in participants_id:
        subset_df = feature_df[feature_df['Participant_ID'] == p_id]
        subset_df.reset_index(inplace=True, drop=True)

        boarder_coordinates = home_coordinates_dict[p_id]
        home_max_lat = boarder_coordinates[1]
        home_min_lat = boarder_coordinates[0]
        home_max_long = boarder_coordinates[3]
        home_min_long = boarder_coordinates[2]

        for idx in subset_df.index:
            coordinates_str = subset_df['location'][idx]
            # print(coordinates_str)
            if type(coordinates_str) != str:
                home = 0
                home_list.append(home)
                continue
            else:
                coordinates = coordinates_str.strip('][').split(', ')

                lat = float(coordinates[0])
                long = float(coordinates[1])
                home = 0

                if lat <= home_max_lat and lat >= home_min_lat and long <= home_max_long and long >= home_min_long:
                    home = 1

                home_list.append(home)
    return home_list



"""
Functionality: Parse the content of .../logs-watch/PromptResponses.log.csv for individual participant into formatted
matrices day by day and save as csv file in the directory .../participant_id@timestudy_com/appended_response.

Usage: This script is intended mainly to be called by uema_all_features_dataframe.py.

Input: python preprocess_promptResponse_raw.py [PARTICIPANT_ID] [microT_root_path] [start_date]
e.g., python preprocess_promptResponse_raw.py aditya4_internal .../MICROT 2020-01-01

Output: csv file for each day of the participant, with each row indicating a prompted question and response
e.g., aditya4_internal_uEMA_****-**-**.csv in .../aditya4_internal@timestudy_com/appended_response
"""
import os
from os import path, sep, makedirs
import csv
import sys
from microt_compliance_matrix.utils.convert_timestamp import *
from glob import glob


def splitMultiLogsinOneRow(row, participant_id):
    rows = []
    id_index = [i for i, s in enumerate(row) if participant_id in s]
    if len(id_index) > 1:

        error_index = id_index[1]
        split1 = row[error_index].replace(participant_id, "")
        split2 = participant_id

        row1 = row[:error_index] + [split1]
        row2 = [split2] + row[error_index + 1:]
        rows.append(row1)
        rows.append(row2)
        if len(id_index) > 2:
            rows = rows[:-1] + splitMultiLogsinOneRow(rows[-1], participant_id)
    else:
        rows.append(row)

    return rows


def extractRow(row):
    new_row = [row[1], row[2], row[10], row[9], row[8]] + row[33:]
    return new_row


def create_temporary_appended_response_files(p_id, intermediate_root_path, feature_save_path, start_date, end_date):
    paticipant_logs_path = intermediate_root_path + sep + "intermediate_file" + sep + p_id

    # step 1: generate date range where date folder exists (sharable code in utils)
    validated_date_list = validate_date(paticipant_logs_path, start_date, end_date)
    if len(validated_date_list) == 0:
        print("Cannot find date folder in data source between {} and {}".format(start_date, end_date))
        return None

    for date in validated_date_list:
        row_list = []
        date_folder_path = paticipant_logs_path + sep + date
        csv_path = sorted(glob(os.path.join(date_folder_path, 'watch_promptresponse_clean*.csv')))
        if len(csv_path) == 0:
            print("No prompt response daily file on {}".format(date))
            continue

        with open(csv_path[0], encoding="utf8") as csv_file:
            csv_reader = csv.reader(csv_file, delimiter=',')
            next(csv_reader)
            for row in csv_reader:
                rows = splitMultiLogsinOneRow(row, p_id)
                for single_row in rows:
                    new_row = extractRow(single_row)
                    row_list.append(new_row)

        df = pd.DataFrame()
        for row in row_list:
            n = 4
            prompt_type = row[0]
            study_mode = row[1]
            completeness = row[2]
            timestamp = row[3]
            time_offset = row[4]
            questions = row[5:]
            questions_grouped_list = [questions[i:i + n] for i in range(0, len(questions), n)]
            grouped_list = [[prompt_type, study_mode, completeness, time_offset, timestamp] + x for x in
                            questions_grouped_list]
            d = pd.DataFrame(grouped_list)
            df = pd.concat([df, d], axis=0)

        df.reset_index(drop=True, inplace=True)
        # write daily response df in long format to a file
        output_save_path = feature_save_path + sep + "compliance_analysis" + sep + "uEMA" + sep + "prompt_response" + sep + p_id + sep + "appended_response"
        if not path.exists(output_save_path):
            makedirs(output_save_path)
        df_temp = df
        df_temp.insert(loc=0, column='ID', value=p_id)

        if df_temp.shape[0] > 0:
            df_temp.columns = ['Participant_ID', 'Type', 'Study_Mode', 'Completion_Status', 'UTC_Offset',
                               'Prompt_Timestamp',
                               'Question_Key', 'Question_Text', 'Prompt_Response', 'Response_Timestamp']
            time_offset_unique_list = df_temp['UTC_Offset'].unique()
            time_offset_unique_list_nonnan = [x for x in time_offset_unique_list if len(x) > 0 ]
            if len(time_offset_unique_list_nonnan) == 1:
                time_offset = time_offset_unique_list_nonnan[0]
            else:
                print("*** multiple time zones for date {}: {}".format(date, time_offset_unique_list_nonnan))
                time_offset = time_offset_unique_list_nonnan[0]

            df_temp['Prompt_Local_Time'] = convert_timestamp_int_list_to_readable_time(df_temp['Prompt_Timestamp'],
                                                                                       time_offset)
            df_temp['Response_Local_Time'] = convert_timestamp_int_list_to_readable_time(df_temp['Response_Timestamp'],
                                                                                         time_offset)
            df_temp = df_temp[
                ['Participant_ID', 'Type', 'Study_Mode', 'Completion_Status', 'UTC_Offset', 'Prompt_Timestamp',
                 'Prompt_Local_Time',
                 'Question_Key', 'Question_Text', 'Prompt_Response', 'Response_Timestamp', 'Response_Local_Time']]
        df_temp.to_csv(
            output_save_path + sep + p_id + "_" + "uEMA_" + date + ".csv",
            index=False)

    return None


if __name__ == "__main__":
    p_id = sys.argv[1]
    intermediate_root_path = sys.argv[2]
    feature_save_path = sys.argv[3]
    start_date = sys.argv[4]
    end_date = sys.argv[5]
    create_temporary_appended_response_files(p_id, intermediate_root_path, feature_save_path, start_date, end_date)



from os import path, sep, makedirs
from glob import glob
import pandas as pd
import numpy as np
from datetime import timedelta
from tqdm import tqdm
from microt_compliance_matrix.utils.validate_date import *


def combine_intermediate_file(participants_ids, intermediate_root_path, feature_save_path, start_date, end_date):
    df_combined = pd.DataFrame()
    for p_id in participants_ids:
        print("\n", p_id)
        participant_logs_path = intermediate_root_path + sep + "intermediate_file" + sep + p_id

        # step 1: generate date range where date folder exists (sharable code in utils)
        validated_date_list = validate_date(participant_logs_path, start_date, end_date)
        if len(validated_date_list) == 0:
            print("Cannot find date folder in data source between {} and {}".format(start_date, end_date))
            return None

        for date in validated_date_list:
            date_folder_path = participant_logs_path + sep + date
            csv_path_list = sorted(glob(os.path.join(date_folder_path, 'phone_app_usage_clean*.csv')))  # file name
            if len(csv_path_list) == 0:
                print("No app usage daily file on {}".format(date))
                continue

            csv_path = csv_path_list[0]
            try:
                df_daily = pd.read_csv(csv_path)
            except:
                print("{} has null file".format(date))
                continue

            if df_daily.shape[0] > 0:
                df_daily['PARTICIPANT_ID'] = [p_id] * df_daily.shape[0]
                df_combined = pd.concat([df_combined, df_daily])

    # # remove error line
    # df_combined = df_combined[df_combined['phone_lock'] != "phone_lock"]
    try:
        df_combined = df_combined[df_combined.APP_EVENT.isin(["KEYGUARD_HIDDEN", "KEYGUARD_SHOWN"])]
    except:
        print("+++++++++++++++++++++++++++")
        print(participants_ids)
        print("+++++++++++++++++++++++++++")
    df_combined.reset_index(inplace=True, drop=True)
    # save a copy of combined file in compliance anaysis folder
    save_path = feature_save_path + sep + "compliance_analysis"  + sep + "uEMA_plus" + sep + "phone_lock"
    if df_combined.shape[0] > 0:
        if not path.exists(save_path):
            makedirs(save_path)

        df_combined.to_csv(save_path + sep + "combined.csv", index=False)

    return df_combined


# def find_closest_time(prompt_time, subset_time_list):
#     i = bisect.bisect_left(subset_time_list, prompt_time)
#     closet_time = min(subset_time_list[max(0, i - 1): i + 2], key=lambda t: abs(prompt_time - t))
#     return closet_time
def find_prev_closest_time(prompt_time, subset_time_list):
    previous_timestamps = subset_time_list[subset_time_list < prompt_time]
    reverse = False
    if len(previous_timestamps) > 0:
        closet_time = previous_timestamps.max()
    else:
        after_timestamps = subset_time_list[subset_time_list > prompt_time]
        closet_time = after_timestamps.min()
        reverse = True
    return closet_time, reverse


def calculate_duration(prompt_time, closest_time):
    # print(prompt_time)
    # print(closest_time)
    # print("\\")
    # dur = (prompt_time-closest_time).seconds
    dur = round((prompt_time-closest_time).seconds / 60, 1)
    if dur > 60:
        dur = 60
    return dur


def match_feature(prompt_feature_df, df_system_combined):
    print("     --- start transformation\n")
    converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S.%f")
    prompt_feature_df['PROMPT_TIMESTAMP'] = list(map(converter, prompt_feature_df['PROMPT_LOCAL_TIME']))
    # try:
    df_system_combined = df_system_combined.dropna(subset=['EVENT_TIME'])
    df_system_combined = df_system_combined[df_system_combined["EVENT_TIME"] != "unknown time"]
    df_system_combined.reset_index(inplace=True, drop=True)

    df_system_combined["EVENT_TIME"] = [x.strip(x.split(' ')[2]).strip(" ") for x in
                                         list(df_system_combined["EVENT_TIME"])]
    df_system_combined['EVENT_TIMESTAMP_LOCAL'] = pd.Series(map(converter, df_system_combined["EVENT_TIME"]))
    df_system_combined['Date'] = [x.date() for x in df_system_combined['EVENT_TIMESTAMP_LOCAL']]
    # except:
    #     print(prompt_feature_df.Participant_ID)
    #     print(df_system_combined.head())
    #     converter = lambda x: datetime.fromtimestamp(x/1000)-timedelta(hours=3)
    #     df_system_combined['EVENT_TIMESTAMP_LOCAL'] = pd.Series(map(converter, df_system_combined["EVENT_TIMESTAMP"]))
    #     df_system_combined['Date'] = df_system_combined['EVENT_TIMESTAMP_LOCAL'].dt.date

    print("     --- start matching\n")
    matched_phone_lock_list = []
    matched_last_usage_list = []
    for idx in tqdm(prompt_feature_df.index):
        p_id = prompt_feature_df['Participant_ID'][idx]
        prompt_time = prompt_feature_df['PROMPT_TIMESTAMP'][idx]
        prompt_date = prompt_time.date()

        df_system_participant = df_system_combined[df_system_combined['PARTICIPANT_ID'] == p_id][
            df_system_combined['Date'].isin([prompt_date-timedelta(1), prompt_date, prompt_date+timedelta(1)])]
        # only keep phone on and off events
        df_system_participant = df_system_participant[df_system_participant.APP_EVENT.isin(["KEYGUARD_HIDDEN", "KEYGUARD_SHOWN"])]

        last_usage = None
        if df_system_participant.shape[0] == 0:
            phone_lock = np.nan
            print("No phone events found")
        else:
            # print(df_system_participant["START_TIMESTAMP"][df_system_participant.shape[0]-20])
            subset_time_list = np.array(df_system_participant["EVENT_TIMESTAMP_LOCAL"])

            closest_time, reverse = find_prev_closest_time(prompt_time, subset_time_list)

            phone_event = list(df_system_participant[df_system_participant['EVENT_TIMESTAMP_LOCAL'] == closest_time][
                                     "APP_EVENT"])[0]


            # to determine phone on/off
            if not reverse:
                if phone_event == "KEYGUARD_HIDDEN":
                    phone_lock = "Phone Unlocked"
                    last_usage = 0
                elif phone_event == "KEYGUARD_SHOWN":
                    phone_lock = "Phone Locked"
                    last_usage = calculate_duration(prompt_time, closest_time)
                else:
                    phone_lock = np.nan
                    last_usage = np.nan
            else:
                if phone_event == "KEYGUARD_HIDDEN":
                    phone_lock = np.nan
                    last_usage = np.nan
                elif phone_event == "KEYGUARD_SHOWN":
                    phone_lock = "Phone Unlocked"
                    last_usage = 0
                else:
                    phone_lock = np.nan
                    last_usage = np.nan

            # print("==   {}".format(phone_lock))

        matched_phone_lock_list.append(phone_lock)
        matched_last_usage_list.append(last_usage)


    return matched_phone_lock_list, matched_last_usage_list


def transform(phone_lock_column):
    # return [1 if x >= 15 else 0 for x in phone_lock_column]
    return phone_lock_column


def create_column(prompt_feature_df, intermediate_root_path, feature_save_path, start_date, end_date):
    print("\n> start generating the feature: phone screen lock ")
    participants_ids = prompt_feature_df['Participant_ID'].unique()

    # Read, parse and combine related intermediate file
    df_combined = combine_intermediate_file(participants_ids, intermediate_root_path, feature_save_path, start_date,
                                            end_date)
    # df_combined = pd.read_csv(r"E:\compliance_analysis\phone_lock\combined.csv")
    # Match the combined parsed intermediate file with prompt feature data frame
    phone_lock_column, last_usage_column = match_feature(prompt_feature_df, df_combined)

    # transform feature
    phone_lock_column_transformed = transform(phone_lock_column)

    return phone_lock_column_transformed, last_usage_column



from . import prompt_response
from . import day_of_the_week
from . import time_of_the_day
from . import days_in_study
from . import compliance
from . import battery_level
from . import study_mode
from . import location
from . import home_location
from . import screen_status
from . import waking_hours
from . import phone_lock
from . import parts_of_waking_hour


from . import validate_date
from . import parse_participants
from . import convert_timestamp
# from . import home_coordinates
from . import extract_participant_id
from . import load_participant_study_info
from. import uncompress_file


from os import sep

def extract_participant_id(intermediate_participant_path):

    participant_id = intermediate_participant_path.split(sep)[-1]

    if not participant_id.endswith("@timestudy_com"):
        raise Exception("Wrong format for input folder path. Needs to be '..\\username@timestudy_com'")

    return participant_id


if __name__ == "__main__":
    intermediate_participant_path = r"E:\data\wocket\Wockets-win32-x64\resources\app\src\srv\MICROT\aditya4_internal@timestudy_com"

    participant_id = extract_participant_id(intermediate_participant_path)
    print("Participant ID  :  {}".format(participant_id))



import pyzipper
import os
import mhealthlab_client.mhlab as mhlab


# def uncompress(zip_file_path, password):
#     password_bytes = password.encode('utf-8')
#     unzip_file_name = zip_file_path.split(os.sep)[-1].split(".zip")[0]
#     zip_folder_path = os.path.abspath(os.path.join(zip_file_path, os.pardir))
#     with pyzipper.AESZipFile(zip_file_path) as zf:
#         zf.extractall(path=zip_folder_path, pwd=password_bytes)
#
#
#     unzip_file_path = os.path.join(zip_folder_path, unzip_file_name)
#
#     return unzip_file_path
def uncompress(zip_file_path, password):
    #password_bytes = password.encode('utf-8')
    unzip_file_name = zip_file_path.split(os.sep)[-1].split(".zip")[0]
    zip_folder_path = os.path.abspath(os.path.join(zip_file_path, os.pardir))
    mhlab.decrypt_file(zip_file_path, zip_folder_path, password.encode())


    unzip_file_path = os.path.join(zip_folder_path, unzip_file_name)

    return unzip_file_path

def delete_unzipped_file(unzip_file_path):
    if os.path.exists(unzip_file_path):
        os.remove(unzip_file_path)
    return


if __name__ == "__main__":
    zip_file_path = r"G:\preprocessed_cluster\intermediate_file\sharpnessnextpouch@timestudy_com\2021-01-01\phone_GPS_clean_2021-01-01.csv.zip"

    password = "TIMEisthenewMICROTStudy-NUUSC"
    unzip_file_path = uncompress(zip_file_path, password)

    # delete_unzipped_file(unzip_file_path)



"""
Output: 1) a list of targeted features for the participant 2) print stats of the targeted features for the participant
e.g., [6,6,6,0,0,0,1,1,1]
"""

from datetime import datetime
from collections import Counter


def print_stats_single_column(feature_name, data):
    print('--------- Stats of Feature: ' + feature_name + ' ---------')
    counter = Counter(data)
    print(sorted(counter.items()))


def create_column(prompt_timestamp_column, print_stats=True):
    print("\n> start generating the feature: day of the week ")
    # map from timestamp to date object
    converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S.%f")
    prompt_dateobject_column = list(map(converter, prompt_timestamp_column))
    # prompt_dateobject_column = list(map(lambda x: datetime.fromtimestamp(x / 1000.0), prompt_timestamp_column))
    # map from date object to weekday
    prompt_weekday_column = list(map(lambda x: x.weekday(), prompt_dateobject_column))

    if print_stats == True:
        print_stats_single_column("weekday", prompt_weekday_column)

    return prompt_weekday_column



import pandas as pd
import numpy as np
from tqdm import tqdm
from datetime import timedelta
import datetime


def match_feature(prompt_feature_df, df_system_combined):
    print("     --- start transformation\n")
    # converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S.%f")
    # prompt_feature_datetime = pd.Series(map(converter, prompt_feature_df['PROMPT_LOCAL_TIME']))
    prompt_feature_df['PROMPT_Hour'] = [int(x.split(' ')[1].split(':')[0]) for x in prompt_feature_df['PROMPT_LOCAL_TIME']]
    promt_date_string = [x.split(" ")[0] for x in prompt_feature_df['PROMPT_LOCAL_TIME']]
    prompt_feature_df['PROMPT_Date'] = [datetime.strptime(x, '%Y-%m-%d').date() for x in promt_date_string]

    df_system_combined = df_system_combined.dropna(subset=['date'])
    df_system_combined.reset_index(inplace=True, drop=True)
    df_system_combined["LOG_TIMESTAMP"] = [datetime.strptime(x, '%m/%d/%Y') for x in df_system_combined['date']]
    df_system_combined['Date'] = df_system_combined['LOG_TIMESTAMP'].dt.date

    print("     --- start matching\n")
    matched_waking_hours_list = []
    matched_sleep_hours_list = []
    for idx in tqdm(prompt_feature_df.index):
        p_id = prompt_feature_df['Participant_ID'][idx]
        prompt_hour = prompt_feature_df['PROMPT_Hour'][idx]
        # print(prompt_hour)

        if prompt_hour > 4:
            prompt_date = prompt_feature_df['PROMPT_Date'][idx]
        else:
            prompt_date = prompt_feature_df['PROMPT_Date'][idx] - timedelta(days=1)

        df_system_participant = df_system_combined[df_system_combined['participant_ID'] == p_id][
            df_system_combined['Date'] == prompt_date]

        if df_system_participant.shape[0] == 0:
            waking_hour = np.nan
            sleep_hour = np.nan
            print("No wake and sleep hour found for {}".format(prompt_date))
        else:
            waking_hour = list(df_system_participant["current_wake_time"])[0]
            sleep_hour = list(df_system_participant["current_sleep_time"])[0]

        matched_waking_hours_list.append(waking_hour)
        matched_sleep_hours_list.append(sleep_hour)

    return matched_waking_hours_list, matched_sleep_hours_list


def transform(waking_hours_column):
    # return [1 if x >= 15 else 0 for x in waking_hours_column]
    return waking_hours_column


def create_column(prompt_feature_df, intermediate_root_path, feature_save_path, start_date, end_date):
    print("\n> start generating the feature: screen status ")
    participants_ids = prompt_feature_df['Participant_ID'].unique()

    # Read, parse and combine related intermediate file
    df_combined = pd.read_csv("/mnt/e/compliance_analysis/report/combined_report.csv")

    # Match the combined parsed intermediate file with prompt feature data frame
    waking_hours_column, sleep_hours_column = match_feature(prompt_feature_df, df_combined)


    return waking_hours_column, sleep_hours_column



"""
Functionality: combine all the csv file in appended_response directory for each participant into one single csv
 and save in the directory .../participant_id@timestudy_com/combined_responses.

Usage: This script is intended mainly to be called by uema_all_features_dataframe.py.

Input: 1). participant_ids, a list of participants' id 2). MICROT_PATH 3). start_date
e.g., [aditya4_internal, jixin_internal]

Output: one combined single csv file for each participant, with each row indicating a prompted question and response
e.g., aditya4_internal_uEMA_combined.csv in .../aditya4_internal@timestudy_com/combined_responses
"""

from os import listdir, path, sep, makedirs
import pandas as pd
from datetime import datetime

def combine_appended_response_for_all_participants(participant_ids, feature_save_path):
    df = pd.DataFrame()
    for p_id in participant_ids:
        print("\nNow Combining for :  ", p_id)
        p_folder_path = feature_save_path + sep + "compliance_analysis" + sep + "uEMA" + sep + "prompt_response" + sep + p_id
        if path.isdir(p_folder_path + sep + "appended_response"):

            paticipant_logs_path = p_folder_path + sep + "appended_response"
            all_csvs = listdir(paticipant_logs_path)
            if len(all_csvs) == 0:
                continue

            for appended_csv in all_csvs:
                df_temp = pd.read_csv(paticipant_logs_path + sep + appended_csv)
                if df_temp.shape[0] > 0:
                    df = pd.concat([df, df_temp])

    # remove error lines
    df = df[df["Prompt_Timestamp"] != -1]
    df.reset_index(drop=True, inplace=True)

    # save the combined csv file
    save_folder_path = feature_save_path + sep + "compliance_analysis"  + sep + "uEMA_plus" + sep + "prompt_response"
    if not path.isdir(save_folder_path):
        makedirs(save_folder_path)
    df.to_csv(save_folder_path + sep + "combined.csv", index=False)

    return df

